'use strict';
const Generator = require('yeoman-generator');
const chalk = require('chalk');
const yosay = require('yosay');
const { kebabCase } = require('lodash');

module.exports = class extends Generator {
  greetUser() {
    this.log(
      yosay(
        'Welcome to the stupendous ' +
          chalk.red('react-redux-ssr') +
          ' generator!'
      )
    );
  }

  getGithubUsernameIfAvailable() {
    this.user.github.username().then(
      username => {
        this.github = username;
      },
      /* eslint handle-callback-err: "off" */
      () => {}
    );
  }

  promptUser() {
    return this.prompt([
      {
        type: 'input',
        name: 'project',
        message: 'Name your project:',
        default: kebabCase(this.appname)
      },
      {
        type: 'input',
        name: 'description',
        message: 'Describe your project:'
      },
      {
        type: 'input',
        name: 'name',
        message: 'Your name:',
        store: true,
        default: this.user.git.name()
      },
      {
        type: 'input',
        name: 'github',
        message: 'Your Github username:',
        store: true,
        default: this.github
      }
    ]).then(inputs => {
      this.inputs = inputs;
    });
  }

  generateProject() {
    const files = {
      'package.json': true,
      'readme.md': true,
      '.babelrc': true,
      '.eslintrc': true,
      '.gitignore': true,
      'docker-compose.yml': true,
      'app.js': 'src/components/app.js',
      'first.js': 'src/components/first.js',
      'second.js': 'src/components/second.js',
      'client.frontend.js': 'src/client.js',
      'server.frontend.js': 'src/server.js',
      'server.js': 'server/index.js',
      'client.dev.webpack.js': 'webpack/client.dev.js',
      'client.prod.webpack.js': 'webpack/client.prod.js',
      'server.dev.webpack.js': 'webpack/server.dev.js',
      'server.prod.webpack.js': 'webpack/server.prod.js'
    };
    for (const key of Object.keys(files)) {
      const source = key;
      const target = files[key] === true ? key : files[key];
      this.fs.copyTpl(
        this.templatePath(source),
        this.destinationPath(target),
        this.inputs
      );
    }
  }

  installPackages() {
    this.npmInstall(
      [
        '@babel/cli',
        '@babel/core',
        '@babel/preset-env',
        '@babel/preset-react',
        '@babel/preset-stage-3',
        'babel-loader@8.0.0-beta.0',
        'babel-plugin-styled-components',
        'babel-plugin-universal-import',
        'express',
        'file-loader',
        'react-dom',
        'react-hot-loader',
        'react-universal-component',
        'react',
        'styled-components',
        'url-loader',
        'webpack-flush-chunks',
        'webpack',
        'write-file-webpack-plugin'
      ],
      { save: true }
    );
    this.npmInstall(
      [
        'babel-eslint@8',
        'eslint-config-standard',
        'eslint-plugin-import',
        'eslint-plugin-node',
        'eslint-plugin-promise',
        'eslint-plugin-react',
        'eslint-plugin-standard',
        'eslint',
        'nodemon',
        'prettier-eslint',
        'prettier',
        'webpack-dev-middleware',
        'webpack-hot-middleware',
        'webpack-hot-server-middleware'
      ],
      { 'save-dev': true }
    );
  }
};
