const { resolve } = require('path')
const {
  HotModuleReplacementPlugin,
  NoEmitOnErrorsPlugin,
  DefinePlugin,
  optimize: { CommonsChunkPlugin }
} = require('webpack')

module.exports = {
  name: 'client',
  target: 'web',
  // devtool: 'source-map',
  devtool: 'eval',
  entry: [
    'webpack-hot-middleware/client?path=/__webpack_hmr&timeout=20000&reload=false&quiet=false&noInfo=false',
    'react-hot-loader/patch',
    resolve(__dirname, '../src/client.js')
  ],
  output: {
    filename: '[name].js',
    chunkFilename: '[name].js',
    path: resolve(__dirname, '../dist/clientFrontend'),
    publicPath: '/static/'
  },
  module: {
    rules: [
      {
        test: /\.jsx?$/,
        exclude: /node_modules/,
        use: 'babel-loader'
      }
    ]
  },
  resolve: {
    extensions: ['.js', '.jsx']
  },
  plugins: [
    new CommonsChunkPlugin({
      names: ['bootstrap'], // puts webpack bootstrap code before chunks
      filename: '[name].js',
      minChunks: Infinity
    }),
    new HotModuleReplacementPlugin(),
    new NoEmitOnErrorsPlugin(),
    new DefinePlugin({
      'process.env': {
        NODE_ENV: JSON.stringify('development')
      }
    })
  ]
}
