const { resolve } = require('path')
const {
  HashedModuleIdsPlugin,
  DefinePlugin,
  optimize: { CommonsChunkPlugin, UglifyJsPlugin }
} = require('webpack')

module.exports = {
  name: 'client',
  target: 'web',
  devtool: 'source-map',
  entry: [resolve(__dirname, '../src/client.js')],
  output: {
    filename: '[name].[chunkhash].js',
    chunkFilename: '[name].[chunkhash].js',
    path: resolve(__dirname, '../dist/clientFrontend'),
    publicPath: '/static/'
  },
  module: {
    rules: [
      {
        test: /\.jsx?$/,
        exclude: /node_modules/,
        use: 'babel-loader'
      }
    ]
  },
  resolve: {
    extensions: ['.js', 'jsx']
  },
  plugins: [
    new CommonsChunkPlugin({
      names: ['bootstrap'], // puts webpack bootstrap code before chunks
      filename: '[name].[chunkhash].js',
      minChunks: Infinity
    }),

    new DefinePlugin({
      'process.env': {
        NODE_ENV: JSON.stringify('production')
      }
    }),
    new UglifyJsPlugin({
      compress: {
        screw_ie8: true,
        warnings: false
      },
      mangle: {
        screw_ie8: true
      },
      output: {
        screw_ie8: true,
        comments: false
      },
      sourceMap: true
    }),
    new HashedModuleIdsPlugin()
  ]
}
