import express from 'express'

import webpack from 'webpack'
import webpackDevMiddleware from 'webpack-dev-middleware'
import webpackHotMiddleware from 'webpack-hot-middleware'
import webpackHotServerMiddleware from 'webpack-hot-server-middleware'

import clientConfig from '../../webpack/client.dev'
import serverConfig from '../../webpack/server.dev'
import clientConfigProd from '../../webpack/client.prod'
import serverConfigProd from '../../webpack/server.prod'

const publicPath = clientConfig.output.publicPath
const outputPath = clientConfig.output.path
const app = express()

let isBuilt = false

const done = () =>
  !isBuilt &&
  app.listen(8080, () => {
    isBuilt = true
    console.log('BUILD COMPLETE -- Listening @ http://localhost:8080')
  })

if (process.env.NODE_ENV === 'development') {
  const compiler = webpack([clientConfig, serverConfig])
  const clientCompiler = compiler.compilers[0]
  const options = { publicPath, stats: { colors: true } }

  app.use(webpackDevMiddleware(compiler, options))
  app.use(webpackHotMiddleware(clientCompiler))
  app.use(webpackHotServerMiddleware(compiler))

  compiler.plugin('done', done)
} else {
  webpack([clientConfigProd, serverConfigProd]).run((err, stats) => {
    if (err) {
      console.error(err)
    }

    const clientStats = stats.toJson().children[0]
    const serverRender = require('../serverFrontend/main.js').default

    app.use(publicPath, express.static(outputPath))
    app.use(serverRender({ clientStats }))

    done()
  })
}
